var gDoorListConfig = {};
var gDoorList = [];
var gDoorMapping = {};
var gSelectedLevelName = UNDEFINED_OPTION;
var gShowOnlyDoors = false;
var gSubdictionary = {};

addEventListener('desiteload', async function(event)
{    
    gTranslator = await createTranslator('/desite.modules/dictionary.json');
    var appLanguage = await desiteAPI.getApplicationLanguage();    
    gSubdictionary = await setCurrentLanguage(gTranslator, appLanguage);
    await desiteAPI.selectionChanged.connect(onSelectionChanged);
    gDoorListConfig = await createDoorListConfig('/desite.modules/doorList.module/doorlistConfig.json');
    createDoorPropertyEdit();
    await createLevelDropdownMenu();
    await showElementsBasedOnFilter();
    await createMissingDoorProperties();
    updateDoorList();
});

async function createDoorListConfig(configfilePath)
{
    var config = {};
    var projectPath = await desiteAPI.getProjectDirectory();
    var filePath = projectPath + configfilePath;
    var fileExists = await desiteAPI.checkIfFileExists(filePath);
    if (fileExists)
        config = JSON.parse(await desiteAPI.readTextFileAsString(filePath, "UTF-8"));
    else
        alert(`ERROR: Couldn't find door list config file '${filePath}'`);
    return config;
}

function updateDoorInfo()
{
    $("#tableDoorInfos").empty();
    var hasEntries = false;
    for (i in gDoorList)
    {
        hasEntries = gDoorList[i].IsSelected;
        if (hasEntries)
            break;
    }
    if (!hasEntries){
        $("#message4").show();
        $("#divDoorInfos").hide();
        return;
    }  

    var bgColors = ["#f7f6f6", "#ffffff"];
    var rowIndex = 0;

    var bg = bgColors[(rowIndex++)%2];
    var innerHtml = '<tr>';
    innerHtml += `<td style="background: ${bg}; left: 0; top: 0; z-index: 1; position: sticky">Name</td>`;  
    innerHtml += `<td style="background: ${bg}; top: 0; position: sticky">Level</td>`;    

    var doorProperties = Object.keys(gDoorListConfig.OptionalProperties);
    for (i in doorProperties)
    {
        var doorProperty = gDoorListConfig.OptionalProperties[doorProperties[i]];
        innerHtml += `<td style="background: ${bg}; top: 0; position: sticky">${doorProperty.DisplayName}</td>`; 
    }
    innerHtml += '</tr>';
    
    for (i in gDoorList)
    {
        var door = gDoorList[i];
        if (!door.IsSelected)
            continue;
        var bg = bgColors[(rowIndex++)%2];
        innerHtml += '<tr>';
        innerHtml += `<td style="background: ${bg}; left: 0; position: sticky">${door.Name}</td>`
        innerHtml += `<td style="background: ${bg}; z-index: -1">${door.Level}</td>`
        var doorProperties = Object.keys(gDoorListConfig.OptionalProperties);
        for (i in doorProperties)
        {
            var value = door.OptionalProperties[doorProperties[i]];
            innerHtml += `<td style="background: ${bg}; z-index: -1">${(value != null ? value : 'n/a')}</td>`
        }
        innerHtml += '</tr>';
    } 
    $('#tableDoorInfos').append(innerHtml);
    $("#message4").hide();
    $("#divDoorInfos").show();    
}

async function createMissingDoorProperties()
{
    var doorProperties = Object.keys(gDoorListConfig.OptionalProperties);
    for (i in doorProperties)
    {
        var doorProperty = gDoorListConfig.OptionalProperties[doorProperties[i]];
        await desiteAPI.createPropertyType(doorProperty.Name, doorProperty.Type, "", true, 1);
    }
}

async function updateDoorList()
{
    gDoorList = [];
    var elementList = await desiteAPI.getAllElements('geometry');        
    elementList = await desiteAPI.filterByProperty(elementList, gDoorListConfig.Door.Name, gDoorListConfig.Door.Type, gDoorListConfig.Door.PropertyValue, false ,'geometry');
    for (i in elementList)
    {
        var element = elementList[i];
        var doorName = await desiteAPI.getPropertyValue(element, "cpName", "xs:string", false);
        var doorLevel = await desiteAPI.getPropertyValue(element, gDoorListConfig.DoorLevel.Name, gDoorListConfig.DoorLevel.Type, false);
        var isSelected = await desiteAPI.isSelected(element);
        var doorOptionalProperties = {};
        var doorProperties = Object.keys(gDoorListConfig.OptionalProperties);
        for (j in doorProperties)
        {
            var doorProperty = gDoorListConfig.OptionalProperties[doorProperties[j]];
            var value = await desiteAPI.getPropertyValue(element, doorProperty.Name, doorProperty.Type, false);
            doorOptionalProperties[doorProperties[j]] = value;
        }
        var door =
        {
            ID: element,
            Name: doorName,
            Level: doorLevel,
            IsSelected: isSelected,
            OptionalProperties: doorOptionalProperties            
        };
        gDoorMapping[door.ID] = i;
        gDoorList.push(door);        
    }    

    gDoorList.sort(function(doorA, doorB)
    {
        return (doorA.Level > doorB.Level ? 1 : (doorA.Level < doorB.Level ? -1 : 0));
    });

    gDoorMapping = {};
    for (i in gDoorList)
    {
        var door = gDoorList[i];
        gDoorMapping[door.ID] = i;
    }

    updateDoorCheckList();
    updateDoorInfo();
    updateDoorPropertyEdit();   
}

async function setDoorSelected(doorID, isSelected)
{    
    var doorIndex = gDoorMapping[doorID];
    if (doorIndex == undefined) 
        return;
    if (gDoorList[doorIndex].IsSelected == isSelected)
        return;
    gDoorList[doorIndex].IsSelected = isSelected;
    $(`#${doorID}`).prop('checked', isSelected);
    await desiteAPI.selectElements(doorID, isSelected, 'geometry');
}

async function createLevelDropdownMenu()
{            
    var undefinedOption = [UNDEFINED_OPTION];
    var levelNameList = await desiteAPI.getPropertyValues('cpRevitLevel', 'xs:string', 0, 'geometry', false); 
    levelNameList = levelNameList.concat(undefinedOption);
    var sortedLevelNameList = levelNameList.sort();
    var dropdown = document.getElementById('dropdownLevel');
    for (i in sortedLevelNameList)
    {
        var levelName = sortedLevelNameList[i];
        var option = document.createElement('option');
        option.textContent = levelName;
        option.value = levelName;
        dropdown.appendChild(option);
    }
}

function createDoorPropertyEdit()
{
    $("#tableDoorEdit").empty();
    var innerHtml = '<tr>'
    innerHtml += `<td width="50%">Name</td>`;
    innerHtml += `<td><input id="Name" style="width: 295px;" readonly></td>`;
    innerHtml += '</tr>';    
    var doorProperties = Object.keys(gDoorListConfig.OptionalProperties);
    for (i in doorProperties)
    {
        var doorProperty = gDoorListConfig.OptionalProperties[doorProperties[i]];
        if (doorProperty.ReadOnly){
            readOnly = 'readonly';
            disabled = 'disabled="disabled"';
        }else{
            readOnly = '';
            disabled = '';
        }
        innerHtml += '<tr>'
        innerHtml += `<td width="50%">${doorProperty.DisplayName}</td>`;
        if (doorProperty.Type == 'xs:boolean')
            innerHtml += `<td><input id="${doorProperties[i]}" onclick="onBooleanEditClicked(this.id)" type="checkbox" ${disabled}/></td>`;
        else if (doorProperty.Type == 'xs:double')
            innerHtml += `<td><input onchange="onStringEditChanged(this.id);" style="width: 295px;" id="${doorProperties[i]}" type="number" min="0" step="0.01" ${readOnly}></td>`;
        else if (doorProperty.Type == 'xs:int')
            innerHtml += `<td><input onchange="onStringEditChanged(this.id);" style="width: 295px;" id="${doorProperties[i]}" type="number" min="0" step="1" ${readOnly}></td>`;
        else
            innerHtml += `<td><input onchange="onStringEditChanged(this.id);" style="width: 295px;" id="${doorProperties[i]}" type="text" ${readOnly}></td>`;
        innerHtml += '</tr>';
    }
    $("#tableDoorEdit").append(innerHtml);    
}

async function updateDoorPropertyEdit()
{    
    var doorProperties = Object.keys(gDoorListConfig.OptionalProperties);

    var nameList = [];
    var optionalProperties = {};
    for (i in doorProperties)
        optionalProperties[doorProperties[i]] = null;

    for (i in gDoorList)
    {
        var door = gDoorList[i];
        if (!door.IsSelected)
            continue;
        nameList.push(door.Name);
        for (j in doorProperties)
        {
            var doorProperty = gDoorListConfig.OptionalProperties[doorProperties[j]];
            var oldValue = optionalProperties[doorProperties[j]];
            var newValue = door.OptionalProperties[doorProperties[j]];
            if (doorProperty.Type == 'xs:boolean')
                optionalProperties[doorProperties[j]] = checkUniqueBoolean(oldValue, newValue ? TriState.True : TriState.False);
            else
                optionalProperties[doorProperties[j]] = checkUniqueString(oldValue, newValue);
        }
    }
    
    if (nameList.length != 0) 
    {        
        $("#message3").hide();
        $("#tableDoorEdit").show();
        if (nameList.length == 1) 
            $("#Name").val(nameList[0]).css('font-style', 'normal');
        else
            $("#Name").val(gSubdictionary["multipleElementsSelected"]).css('font-style', 'italic');            
        
        for (i in doorProperties) 
        {
            var doorProperty = gDoorListConfig.OptionalProperties[doorProperties[i]];
            var value = optionalProperties[doorProperties[i]];
            if (doorProperty.Type == 'xs:boolean') 
            {
                $(`#${doorProperties[i]}`).prop('indeterminate', value == TriState.Indeterminate);
                $(`#${doorProperties[i]}`).prop('checked', value == TriState.True);
            }
            else 
            {
                $(`#${doorProperties[i]}`).val(value);
            }
        }        
    } 
    else 
    {
        $("#message3").show();
        $("#tableDoorEdit").hide();
    }
}

async function showElementsBasedOnFilter()
{        
    var elementList = await desiteAPI.getAllElements('geometry');
    if (gSelectedLevelName != UNDEFINED_OPTION)
    {
        var levelNameFilter = `"${gSelectedLevelName}"`;
        elementList = await desiteAPI.filterByProperty(elementList, 'cpRevitLevel', 'xs:string', levelNameFilter, true ,'geometry');
    }    
    if (gShowOnlyDoors)
    {        
        elementList = await desiteAPI.filterByProperty(elementList, 'cpRevitCategory', 'xs:string', 'Doors', true ,'geometry');
    }
    await desiteAPI.showElementsOnly(elementList);
}

function updateDoorCheckList()
{
    $("#tableDoorList").empty();
    for (i in gDoorList)
    {
        var door = gDoorList[i];
        if (gSelectedLevelName != UNDEFINED_OPTION && door.Level != gSelectedLevelName)
            continue;
        var checked = (door.IsSelected ? 'checked' : '');
        $('#tableDoorList').append(`<tr><td><input type="checkbox" id="${door.ID}" onclick="onDoorSelectionChanged(this.id)" ${checked}/></td><td>${door.Name} - ${door.Level}</td></tr>`);        
    } 

    if ($('#tableDoorList tr').length != 0){
        $("#message1").show();
        $("#message2").hide();        
        $("#divDoorList").show();
    }else {
        $("#message1").hide();
        $("#message2").show();
        $("#divDoorList").hide();
    }     
}

async function onBooleanEditClicked(id)
{
    var isChecked = $(`#${id}`).is(":checked");
    var doorProperty = gDoorListConfig.OptionalProperties[id];
    for (i in gDoorList)
    {
        var door = gDoorList[i];
        if (!door.IsSelected)
            continue;
        desiteAPI.setPropertyValue(door.ID, doorProperty.Name, doorProperty.Type, isChecked);
        door.OptionalProperties[id] = isChecked;
    }
    updateDoorInfo();
    updateDoorPropertyEdit();   
}

async function onStringEditChanged(id)
{
    var value = $(`#${id}`).val();
    var doorProperty = gDoorListConfig.OptionalProperties[id];
    for (i in gDoorList)
    {
        var door = gDoorList[i];
        if (!door.IsSelected)
            continue;
        desiteAPI.setPropertyValue(door.ID, doorProperty.Name, doorProperty.Type, value);
        door.OptionalProperties[id] = value;
    }
    updateDoorInfo();
    updateDoorPropertyEdit();   
}

async function onLevelDropdownMenuChanged() 
{
    var dropdown = document.getElementById('dropdownLevel');
    var selectedIndex = dropdown.selectedIndex;
    gSelectedLevelName = dropdown.options[selectedIndex].value;
    showElementsBasedOnFilter();    
    updateDoorCheckList();
}

async function onDoorCheckboxClicked()
{
    gShowOnlyDoors = $("#checkboxDoor").is(":checked");
    showElementsBasedOnFilter();
}

async function onDoorSelectionChanged(id)
{
    var isChecked = $(`#${id}`).is(":checked");
    await setDoorSelected(id, isChecked);
}

async function onSelectionChanged() 
{
    var selectElementList = await desiteAPI.getSelectedElements('geometry');
    var selectElementSet = new Set(selectElementList);
    for (i in gDoorList)
    {
        var door = gDoorList[i];
        var isSelected = selectElementSet.has(door.ID);
        await setDoorSelected(door.ID, isSelected);
    }
    updateDoorInfo();
    updateDoorPropertyEdit();   
}

async function onPrintClicked()
{
    await desiteMD.setPageMargins(20,20,20,20);
    await desiteMD.setOrientation(1);   
    await desiteMD.setPaperSize(0);
    var textBlocks =
    {
        content: $('#tableDoorInfos').html()
    };
    var filePath = await desiteAPI.getProjectDirectory() + '/desite.modules/samples.printTemplate.html';
    await desiteMD.createPdfByTemplate(filePath,'', textBlocks);
}