const DEFAULT_DATE_FORMAT = 'dd.mm.yy';
const DEFAULT_START_DAY = 1; // To convert from US format where sunday is the first day of the week to European format where monday is the first day of the week.
const DEFAULT_SHOW_WEEK = true;
const DEFAULT_SHOW_OTHER_MONTHS = true;

function datepickerOptionsGerman()
{
    var dateOptions =
    {
        dateFormat: DEFAULT_DATE_FORMAT,
        showWeek: DEFAULT_SHOW_WEEK,
        showOtherMonths: DEFAULT_SHOW_OTHER_MONTHS,
        firstDay: DEFAULT_START_DAY, 
        weekHeader: 'KW',
        monthNames: ["Januar", "Februar", "März", "April", "Mai", "Juni", "Juli", "August", "September", "Oktober", "November", "Dezember"],
        monthNamesShort: ["Jan", "Feb", "Mär", "Apr", "Mai", "Jun", "Jul", "Aug", "Sep", "Okt", "Nov", "Dez"],
        dayNames: ["Sonntag", "Montag", "Dienstag", "Mittwoch", "Donnerstag", "Freitag", "Samstag"],
        dayNamesShort: ["So", "Mo", "Di", "Mi", "Do", "Fr", "Sa"],
        dayNamesMin:["So", "Mo", "Di", "Mi", "Do", "Fr", "Sa"]
    };
    return dateOptions;
}

function datepickerOptionsEnglish()
{
    var dateOptions =
    {
        dateFormat: DEFAULT_DATE_FORMAT,
        showWeek: DEFAULT_SHOW_WEEK,
        showOtherMonths: DEFAULT_SHOW_OTHER_MONTHS,
        firstDay: DEFAULT_START_DAY
    };
    return dateOptions;
}

async function createTranslator(dictionaryFilePath)
{
    var projectPath = await desiteAPI.getProjectDirectory();
    var filePath = projectPath + dictionaryFilePath;
    var fileExists = await desiteAPI.checkIfFileExists(filePath);
    var translator = {};
    if (fileExists)
    {
        var dictionary = JSON.parse(await desiteAPI.readTextFileAsString(filePath, "UTF-8"));     
        translator =
        {
            dictionary: dictionary,
            currentLanguage: 'undefined'
        };
    }    
    else
    {
        alert(`ERROR: Couldn't find dictionary file '${dictionaryFilePath}'`);
    }
    return translator;
}

async function setCurrentLanguage(translator, currentLanguage)
{        
    var dictionary = translator.dictionary;    
    var language = 'en';
    var languages = dictionary.languages;
    for (i in languages)
    {
        if (languages[i] == currentLanguage)
        {
            language = currentLanguage;
            break;
        }
    }
    var translations = dictionary.translations;
    var subdictionary = {};
    for (i in translations)
    {
        var translation = translations[i];
        var key = Object.keys(translation)[0];
        var element = document.getElementById("translate:" + key);
        value = (language in translation[key] ? translation[key][language] : "missing");
        if(element)
        {
            
            if (element.type == "button")
                element.value = value;
            else
                element.innerHTML = value;
        }
        else
        {
            subdictionary[key] = value;
        }
    }
    translator.currentLanguage = language;
    return subdictionary;
}

function getCurrentLanguage(translator)
{
    return translator.currentLanguage;
}