/*
You can find the Open Layers API documentation here.
https://openlayers.org/en/latest/apidoc/
*/

var gTranslator = {};
var gMap;
var gOverlay;
var gProjection;
var gPropectionType;
var gSelectControl;
var gDefaultLocation = { Latitude: 50, Longitude: 5 };
var gDefaultZoom = 5;
var gShowBuilding = true;
var gShowDocuments = true;
var gFeatures = [];
var gBuildingFeature;
var gDocumentsFeatures = [];

addEventListener('desiteload', async function(event)
{          
    gTranslator = await createTranslator('/desite.modules/dictionary.json');
    var appLanguage = await desiteAPI.getApplicationLanguage();    
    setCurrentLanguage(gTranslator, appLanguage);
    createMap();    
});

async function updateLocationsOnMap()
{
    for (i in gFeatures)
    {
        var feature = gFeatures[i];
        if (feature.popup)
            destroyPopup(feature);
    }
    gOverlay.removeAllFeatures();

    var visibleFeatures = [];
    if (gShowBuilding)
        visibleFeatures.push(gBuildingFeature); 
    if (gShowDocuments)
        visibleFeatures = visibleFeatures.concat(gDocumentsFeatures);        
    gOverlay.addFeatures(visibleFeatures);
}

async function createMap()
{
    gMap = new OpenLayers.Map("mapdiv");
    gMap.addLayer(new OpenLayers.Layer.OSM());    
    gOverlay = new OpenLayers.Layer.Vector("Overlay");
    gMap.addLayer(gOverlay);
    featureSelect =
    {
        onSelect: createPopup,
        onUnselect: destroyPopup
    };
    gSelectControl =
    {
        selector: new OpenLayers.Control.SelectFeature(gOverlay, featureSelect)
    };
    gMap.addControl(gSelectControl['selector']);
    gSelectControl['selector'].activate();

    gPropectionType =  new OpenLayers.Projection("EPSG:4326"); // WGS 1984 projection
    gProjection = gMap.getProjectionObject(); // The map projection (Spherical Mercator)
    
    var defaultLocation = new OpenLayers.LonLat(gDefaultLocation.Longitude, gDefaultLocation.Latitude).transform(gPropectionType, gProjection);
    gMap.setCenter(defaultLocation, gDefaultZoom);

    var buildingLocation = await desiteAPI.getProjectLocation();
    gBuildingFeature = new OpenLayers.Feature.Vector(
        new OpenLayers.Geometry.Point(buildingLocation.lon, buildingLocation.lat).transform(gPropectionType, gProjection),
        {
            description: ("Lat: " + buildingLocation.lat.toFixed(5) + ", Lon: "+ buildingLocation.lon.toFixed(5))
        },
        {
            externalGraphic: 'img/home_32x32.png',
            graphicHeight: 32,
            graphicWidth: 32,
            graphicXOffset:-12,
            graphicYOffset:-25
        }
        );   
    gBuildingFeature.attributes.thumbnail = '<img src="img/waldstrasse.png" style="width:128px;height:128px;"/>';
    gFeatures.push(gBuildingFeature);

    var documents = await desiteAPI.getAllElements('documents');
    for (i in documents)
    {
        var doc = documents[i];
        var locationStr = await desiteAPI.getPropertyValue(doc,'GpsLocationJSON','xs:string');
        if(!locationStr)
            continue;
        var location = JSON.parse(locationStr);
        var url = await desiteAPI.getPropertyValue(doc,'GlobalURL','xs:anyURI');
        var feature = new OpenLayers.Feature.Vector(
            new OpenLayers.Geometry.Point(location["Longitude"], location["Latitude"]).transform(gPropectionType, gProjection),
            {
                description: ("Lat: " + location.Latitude.toFixed(5) + ", Lon: "+ location.Longitude.toFixed(5))
            },
            {
                externalGraphic: 'img/pin.png',
                graphicHeight: 32,
                graphicWidth: 32,
                graphicXOffset:-12,
                graphicYOffset:-25
            }
            );
        feature.attributes.thumbnail = '<img src="'+ url +'" style="width:128px;height:128px;"/>';
        gDocumentsFeatures.push(feature);
        gFeatures.push(feature);
    } 

    updateLocationsOnMap();
}

function createPopup(feature)
{
    var content = '<div class="markerContent">' + feature.attributes.thumbnail + '<br>' + feature.attributes.description + '</div>';
    feature.popup = new OpenLayers.Popup.FramedCloud(
        "xxxId",
        feature.geometry.getBounds().getCenterLonLat(),
        null,
        content,
        null,
        true,
        function() { gSelectControl['selector'].unselectAll(); }
        );
    gMap.addPopup(feature.popup);
}

function destroyPopup(feature)
{
    feature.popup.destroy();
    feature.popup = null;
}

function onResetLocationClicked()
{
    var defaultLocation = new OpenLayers.LonLat(gDefaultLocation.Longitude, gDefaultLocation.Latitude).transform(gPropectionType, gProjection);
    gMap.setCenter(defaultLocation, gDefaultZoom);
}

function onShowLocationClicked()
{
    if (!gShowBuilding && !gShowDocuments)
        return;

    var extent = gOverlay.getDataExtent();
    gMap.zoomToExtent(extent);
}

function onShowBuildingClicked()
{
    gShowBuilding = $("#showBuildingCheckbox").is(":checked");
    updateLocationsOnMap();
}

function onShowDocumentsClicked()
{
    gShowDocuments = $("#showDocumentsCheckbox").is(":checked");
    updateLocationsOnMap();
}