var gTranslator = {};
var gRoombookConfig = {};
var gRoomList = [];
var gRoomMapping = {};
var gSelectedLevelName = UNDEFINED_OPTION;
var gSelectedRoomTypeName = UNDEFINED_OPTION;
var gSubdictionary = {};

addEventListener('desiteload', async function(event)
{        
    await desiteAPI.selectionChanged.connect(onSelectionChanged);
    gTranslator = await createTranslator('/desite.modules/dictionary.json');
    var appLanguage = await desiteAPI.getApplicationLanguage();
    gSubdictionary = await setCurrentLanguage(gTranslator, appLanguage);
    gRoombookConfig = await createRoombookConfig('/desite.modules/roombook.module/roombookConfig.json');
    createRoomPropertyEdit();    
    await createLevelDropdownMenu();
    await createRoomTypeDropdownMenu();
    await createMissingRoomProperties();        
    updateRoomList();
    showElementsBasedOnFilter();    
});            

async function createRoombookConfig(configfilePath)
{
    var config = {};
    var projectPath = await desiteAPI.getProjectDirectory();
    var filePath = projectPath + configfilePath;
    var fileExists = await desiteAPI.checkIfFileExists(filePath);
    if (fileExists)
        config = JSON.parse(await desiteAPI.readTextFileAsString(filePath, "UTF-8"));
    else
        alert(`ERROR: Couldn't find roombook config file '${filePath}'`);
    return config;
}

async function createMissingRoomProperties()
{
    var roomProperties = Object.keys(gRoombookConfig.OptionalProperties);
    for (i in roomProperties)
    {
        var roomProperty = gRoombookConfig.OptionalProperties[roomProperties[i]];
        await desiteAPI.createPropertyType(roomProperty.Name, roomProperty.Type, "", true, 1);
    }
}

async function createLevelDropdownMenu()
{            
    var undefinedOption = [UNDEFINED_OPTION];
    var roomLevel = gRoombookConfig.RoomLevel;
    var roomLevelNameList = await desiteAPI.getPropertyValues(roomLevel.Name, roomLevel.Type, 0, "geometry", false);
    roomLevelNameList = roomLevelNameList.concat(undefinedOption);
    var sortedRoomLevelNameList = roomLevelNameList.sort();
    var dropdown = document.getElementById("dropdownLevel");
    for (i in sortedRoomLevelNameList)
    {
        var levelName = sortedRoomLevelNameList[i];
        var option = document.createElement("option");
        option.textContent = levelName;
        option.value = levelName;
        dropdown.appendChild(option);
    }
}

async function createRoomTypeDropdownMenu()
{            
    var undefinedOption = [UNDEFINED_OPTION];
    var roomType = gRoombookConfig.RoomType;
    var roomTypeNameList = await desiteAPI.getPropertyValues(roomType.Name, roomType.Type, 0, "geometry", false);
    roomTypeNameList = roomTypeNameList.concat(undefinedOption);
    var sortedRoomTypeNameList = roomTypeNameList.sort();
    var dropdown = document.getElementById("dropdownRoomType");
    for (i in sortedRoomTypeNameList)
    {
        var roomTypeName = sortedRoomTypeNameList[i];
        var option = document.createElement("option");
        option.textContent = roomTypeName;
        option.value = roomTypeName;
        dropdown.appendChild(option);
    }
}

function createRoomPropertyEdit()
{
    $("#tableRoomEdit").empty();
    var innerHtml = '<tr>'
    innerHtml += `<td width="50%">Name</td>`;
    innerHtml += `<td><input id="Name" style="width: 295px;" readonly></td>`;
    innerHtml += '</tr>';    
    var roomProperties = Object.keys(gRoombookConfig.OptionalProperties);
    for (i in roomProperties)
    {
        var roomProperty = gRoombookConfig.OptionalProperties[roomProperties[i]];
        if(roomProperty.ReadOnly){
            //continue;
            readOnly = 'readonly';
            disabled = 'disabled="disabled"';
        }else {
            readOnly = '';
            disabled = '';
        }
        
        innerHtml += '<tr>'
        innerHtml += `<td width="50%">${roomProperty.DisplayName}</td>`;
        if (roomProperty.Type == 'xs:boolean')
            innerHtml += `<td><input id="${roomProperties[i]}" onclick="onBooleanEditClicked(this.id)" type="checkbox" ${disabled}/></td>`;
        else if (roomProperty.Type == 'xs:double')
            innerHtml += `<td><input onchange="onStringEditChanged(this.id);" style="width: 295px;" id="${roomProperties[i]}" type="number" min="0" step="0.01" ${readOnly}></td>`;
        else if (roomProperty.Type == 'xs:int')
            innerHtml += `<td><input onchange="onStringEditChanged(this.id);" style="width: 295px;" id="${roomProperties[i]}" type="number" min="0" step="1" ${readOnly}></td>`;
        else
            innerHtml += `<td><input onchange="onStringEditChanged(this.id);" style="width: 295px;" id="${roomProperties[i]}" type="text" ${readOnly}></td>`;
        innerHtml += '</tr>';
    }
    $("#tableRoomEdit").append(innerHtml);    
}

async function updateRoomPropertyEdit()
{    
    var roomProperties = Object.keys(gRoombookConfig.OptionalProperties);

    var nameList = [];
    var optionalProperties = {};
    for (i in roomProperties)
        optionalProperties[roomProperties[i]] = null;

    for (i in gRoomList)
    {
        var room = gRoomList[i];
        if (!room.IsSelected)
            continue;
        nameList.push(room.Name);
        for (j in roomProperties)
        {
            var roomProperty = gRoombookConfig.OptionalProperties[roomProperties[j]];
            //if(roomProperty.ReadOnly)
            //    continue;
            var oldValue = optionalProperties[roomProperties[j]];
            var newValue = room.OptionalProperties[roomProperties[j]];
            if (roomProperty.Type == 'xs:boolean')
                optionalProperties[roomProperties[j]] = checkUniqueBoolean(oldValue, newValue ? TriState.True : TriState.False);
            else
                optionalProperties[roomProperties[j]] = checkUniqueString(oldValue, newValue);
        }
    }

    if (nameList.length != 0) 
    {
        $("#message3").hide();
        $("#tableRoomEdit").show();
        if (nameList.length == 1) 
            $("#Name").val(nameList[0]).css('font-style', 'normal');
        else
            $("#Name").val(gSubdictionary["multipleElementsSelected"]).css('font-style', 'italic');

        for (i in roomProperties) {
            var roomProperty = gRoombookConfig.OptionalProperties[roomProperties[i]];
            var value = optionalProperties[roomProperties[i]];
            if (roomProperty.Type == 'xs:boolean') {
                $(`#${roomProperties[i]}`).prop('indeterminate', value == TriState.Indeterminate);
                $(`#${roomProperties[i]}`).prop('checked', value == TriState.True);
            }
            else {
                $(`#${roomProperties[i]}`).val(value);
            }
        }
    } 
    else 
    {
        $("#message3").show();
        $("#tableRoomEdit").hide();
    }    
}

function updateRoomInfo()
{
    $("#tableRoomInfos").empty();
    var hasEntries = false;
    for (i in gRoomList)
    {
        hasEntries = gRoomList[i].IsSelected;
        if (hasEntries)
            break;
    }
    if (!hasEntries){
        $("#message4").show();
        $("#divRoomInfos").hide();
        return;
    }  
    
    var bgColors = ["#f7f6f6", "#ffffff"];
    var rowIndex = 0;

    var bg = bgColors[(rowIndex++)%2];
    var innerHtml = '<tr>';
    innerHtml += `<td style="background: ${bg}; z-index: 1; left: 0; top: 0; position: sticky">Name</td>`;  
    innerHtml += `<td style="background: ${bg}; top: 0; position: sticky">Level</td>`;                      
    innerHtml += `<td style="background: ${bg}; top: 0; position: sticky">Type</td>`;                       

    var roomProperties = Object.keys(gRoombookConfig.OptionalProperties);
    for (i in roomProperties)
    {
        var roomProperty = gRoombookConfig.OptionalProperties[roomProperties[i]];
        innerHtml += `<td style="background: ${bg}; top:0; position: sticky">${roomProperty.DisplayName}</td>`;  
    }
    innerHtml += '</tr>';
    
    for (i in gRoomList)
    {
        var room = gRoomList[i];
        if (!room.IsSelected)
            continue;
        var bg = bgColors[(rowIndex++)%2];
        innerHtml += '<tr>';
        innerHtml += `<td style="background: ${bg}; left: 0; position: sticky">${room.Name}</td>`
        innerHtml += `<td style="background: ${bg}; z-index: -1">${room.Level}</td>`
        innerHtml += `<td style="background: ${bg}; z-index: -1">${room.Type}</td>`
        var roomProperties = Object.keys(gRoombookConfig.OptionalProperties);
        for (i in roomProperties)
        {
            var value = room.OptionalProperties[roomProperties[i]];
            innerHtml += `<td style="background: ${bg}; z-index: -1">${(value != null ? value : 'n/a')}</td>`
        }
        innerHtml += '</tr>';
    }    

    $('#tableRoomInfos').append(innerHtml);
    $("#message4").hide();
    $("#divRoomInfos").show();  
}

async function updateRoomList()
{    
    gRoomList = [];
    var elementList = await desiteAPI.getAllElements('geometry');        
    elementList = await desiteAPI.filterByProperty(elementList, gRoombookConfig.Room.Name, gRoombookConfig.Room.Type, gRoombookConfig.Room.PropertyValue, false ,'geometry');
    for (i in elementList)
    {
        var element = elementList[i];
        var roomName = await desiteAPI.getPropertyValue(element, "cpName", "xs:string", false);
        var roomLevel = await desiteAPI.getPropertyValue(element, gRoombookConfig.RoomLevel.Name, gRoombookConfig.RoomLevel.Type, false);
        var roomType = await desiteAPI.getPropertyValue(element, gRoombookConfig.RoomType.Name, gRoombookConfig.RoomType.Type, false);
        var isSelected = await desiteAPI.isSelected(element);
        var roomOptionalProperties = {};
        var roomProperties = Object.keys(gRoombookConfig.OptionalProperties);
        for (j in roomProperties)
        {
            var roomProperty = gRoombookConfig.OptionalProperties[roomProperties[j]];
            var value = await desiteAPI.getPropertyValue(element, roomProperty.Name, roomProperty.Type, false);
            roomOptionalProperties[roomProperties[j]] = value;
        }
        var room =
        {
            ID: element,
            Name: roomName,
            Level: roomLevel,
            Type: roomType,
            IsSelected: isSelected,
            OptionalProperties: roomOptionalProperties            
        };        
        gRoomList.push(room);
    }    

    gRoomList.sort(function(roomA, roomB)
    {
        return (roomA.Level > roomB.Level ? 1 : (roomA.Level < roomB.Level ? -1 : 0));
    });

    gRoomMapping = {};
    for (i in gRoomList)
    {
        var room = gRoomList[i];
        gRoomMapping[room.ID] = i;
    }

    updateRoomCheckList();
    updateRoomInfo();    
    updateRoomPropertyEdit();
}

async function setRoomSelected(roomID, isSelected)
{    
    var roomIndex = gRoomMapping[roomID];
    if(roomIndex == undefined)
        return;
    if(gRoomList[roomIndex].IsSelected == isSelected)
        return;
    gRoomList[roomIndex].IsSelected = isSelected;
    $(`#${roomID}`).prop('checked', isSelected);
    await desiteAPI.selectElements(roomID, isSelected, 'geometry');
}

async function onRoomSelectionChanged(id)
{
    var isChecked = $(`#${id}`).is(":checked");
    await setRoomSelected(id, isChecked);
    updateRoomInfo();
    updateRoomPropertyEdit();
}

async function onSelectionChanged()
{
    var selectElementList = await desiteAPI.getSelectedElements('geometry');
    var selectElementSet = new Set(selectElementList);
    for (i in gRoomList)
    {
        var room = gRoomList[i];
        var isSelected = selectElementSet.has(room.ID);
        await setRoomSelected(room.ID, isSelected);
    }
    updateRoomInfo();
    updateRoomPropertyEdit();
}

async function onBooleanEditClicked(id)
{
    var isChecked = $(`#${id}`).is(":checked");
    var roomProperty = gRoombookConfig.OptionalProperties[id];
    for (i in gRoomList)
    {
        var room = gRoomList[i];
        if (!room.IsSelected)
            continue;
        desiteAPI.setPropertyValue(room.ID, roomProperty.Name, roomProperty.Type, isChecked);
        room.OptionalProperties[id] = isChecked;
    }
    updateRoomInfo();
    updateRoomPropertyEdit();   
}

async function onStringEditChanged(id)
{
    var value = $(`#${id}`).val();
    var roomProperty = gRoombookConfig.OptionalProperties[id];
    for (i in gRoomList)
    {
        var room = gRoomList[i];
        if (!room.IsSelected)
            continue;
        desiteAPI.setPropertyValue(room.ID, roomProperty.Name, roomProperty.Type, value);
        room.OptionalProperties[id] = value;
    }
    updateRoomInfo();
    updateRoomPropertyEdit();   
}

function updateRoomCheckList()
{
    $("#tableRoomList").empty();
    for (i in gRoomList)
    {
        var room = gRoomList[i];
        if(gSelectedLevelName != UNDEFINED_OPTION && room.Level != gSelectedLevelName)
            continue;
        if(gSelectedRoomTypeName != UNDEFINED_OPTION && room.Type != gSelectedRoomTypeName)
            continue;
        var checked = (room.IsSelected ? 'checked' : '');        
        $('#tableRoomList').append(`<tr><td><input type="checkbox" id="${room.ID}" onclick="onRoomSelectionChanged(this.id)" ${checked}/></td><td>${room.Name} - ${room.Level} - ${room.Type}</td></tr>`);        
    } 
    
    if ($('#tableRoomList tr').length != 0){
        $("#message2").hide();
        $("#message1").show();
        $("#divRoomList").show();
    }else {
        $("#message1").hide();
        $("#message2").show();
        $("#divRoomList").hide();
    }
}

async function showElementsBasedOnFilter()
{
    var elementList = await desiteAPI.getAllElements('geometry');        
    elementList = await desiteAPI.filterByProperty(elementList, gRoombookConfig.Room.Name, gRoombookConfig.Room.Type, gRoombookConfig.Room.PropertyValue, false ,'geometry');
    if (gSelectedLevelName != UNDEFINED_OPTION)
    {
        var levelNameFilter = `"${gSelectedLevelName}"`;
        elementList = await desiteAPI.filterByProperty(elementList, gRoombookConfig.RoomLevel.Name, gRoombookConfig.RoomLevel.Type, levelNameFilter, false ,'geometry');
    }
    if (gSelectedRoomTypeName != UNDEFINED_OPTION)
    {
        var roomTypeNameFilter = `"${gSelectedRoomTypeName}"`;
        elementList = await desiteAPI.filterByProperty(elementList, gRoombookConfig.RoomType.Name, gRoombookConfig.RoomType.Type, roomTypeNameFilter, false ,'geometry');
    }
    await desiteAPI.showElementsOnly(elementList);
}

async function onDropdownLevelChanged()
{
    var selectedOption = document.getElementById('dropdownLevel');
    var selectedIndex = selectedOption.selectedIndex;
    gSelectedLevelName = selectedOption.options[selectedIndex].value;
    showElementsBasedOnFilter();   
    updateRoomCheckList();
}

async function onDropdownRoomTypeChanged()
{
    var selectedOption = document.getElementById('dropdownRoomType');
    var selectedIndex = selectedOption.selectedIndex;
    gSelectedRoomTypeName = selectedOption.options[selectedIndex].value;
    showElementsBasedOnFilter();   
    updateRoomCheckList();
}

async function onPrintClicked()
{
    await desiteMD.setPageMargins(20,20,20,20);
    await desiteMD.setOrientation(1);   
    await desiteMD.setPaperSize(0);
    var textBlocks =
    {
        content: $('#tableRoomInfos').html()
    };
    var filePath = await desiteAPI.getProjectDirectory() + '/desite.modules/samples.printTemplate.html';
    await desiteMD.createPdfByTemplate(filePath,'', textBlocks);
}