const StatusType = 
{
    None: 0,
    All: 1,
    Overdue: 2,
    OnTime: 3
};

var gTranslator = {};
var gSubdictionary = {};
var gSelectedLevelName = UNDEFINED_OPTION;
var gSelectedCategoryName = UNDEFINED_OPTION;
var gShowOnlyStatus = false;
var gCurrentStatus = StatusType.None;
var gOverdueMaterialName = "Overdue";
var gOnTimeMaterialName = "On time";

addEventListener('desiteload', async function(event)
{    
    gTranslator = await createTranslator('/desite.modules/dictionary.json');
    var appLanguage = await desiteAPI.getApplicationLanguage();
    gSubdictionary = await setCurrentLanguage(gTranslator, appLanguage);
    setDatepickerLanguage(appLanguage);
    await desiteAPI.selectionChanged.connect(onSelectionChanged);
    await createMaterials();
    await createLevelDropdownMenu();
    await createCategoryDropdownMenu();    
    createStatusDropdownMenu();        
    await desiteAPI.resetMaterials(true);
    await showElementsBasedOnFilter();           
    await updateSelectedElementInfo();        
});

function onSelectionChanged()
{
    updateSelectedElementInfo();
}

/*
Creates dropdown menu options for available categories in the model, sorted in ascending order
*/
async function createCategoryDropdownMenu()
{
    var undefinedOption = [UNDEFINED_OPTION];
    var categoryNameList = await desiteAPI.getPropertyValues("cpRevitCategory", "xs:string", 0 ,"geometry", false);
    categoryNameList = categoryNameList.concat(undefinedOption);
    var sortedCategoryNameList = categoryNameList.sort();
    var dropdown = document.getElementById("dropdownCategory");
    for (i in sortedCategoryNameList)
    {
        var categoryName = sortedCategoryNameList[i];
        var option = document.createElement("option");
        option.textContent = categoryName;
        option.value =  categoryName;
        dropdown.appendChild(option);
    }
}

/*
Creates dropdown menu options for available levels in the model, sorted in ascending order.
*/
async function createLevelDropdownMenu()
{            
    var undefinedOption = [UNDEFINED_OPTION];
    var levelNameList = await desiteAPI.getPropertyValues("cpRevitLevel", "xs:string", 0, "geometry", false);
    levelNameList = levelNameList.concat(undefinedOption);
    var sortedLevelNameList = levelNameList.sort();
    var dropdown = document.getElementById("dropdownLevel");
    for (i in sortedLevelNameList)
    {
        var levelName = sortedLevelNameList[i];
        var option = document.createElement("option");
        option.textContent = levelName;
        option.value = levelName;
        dropdown.appendChild(option);
    }
}

function createStatusDropdownMenu()
{            
    var options = [UNDEFINED_OPTION, "All", "Overdue", "On Time"];
    var optionValues = [StatusType.None, StatusType.All, StatusType.Overdue, StatusType.OnTime];
    var dropdown = document.getElementById("dropdownStatus");
    for (i in options)
    {
        var option = document.createElement("option");
        option.textContent = options[i];
        option.value = optionValues[i];
        dropdown.appendChild(option);
    }
}

/*
Updates the GUI for the first linked activity of the currently selected element.
Currently only a single element and it's first linked activity are supported.
*/
async function updateSelectedElementInfo()
{
    var name = '';
    var linkedTasksCount = '';
    var activityName = '';
    var startDate = '';
    var finishDate = '';
    var actualFinishDate = '';
    $('#inputDateStart').prop('disabled', true);
    $('#inputDateFinish').prop('disabled', true);
    $('#inputDateActualFinish').prop('disabled', true);

    var selectedElements = await desiteAPI.getSelectedElements();
    if (selectedElements.length == 0){
        $("#message").show();
        $("#tableScheduleEdit").hide();
    }else{
        $("#message").hide();
        $("#tableScheduleEdit").show();

        if (selectedElements.length == 1) {
            var firstSelectedElement = selectedElements[0];
            var name = await desiteAPI.getPropertyValue(firstSelectedElement, 'cpName', 'xs:string');
            var linkedActivities = await desiteAPI.getLinkedObjects(firstSelectedElement, 'activities', 'geometry');
            if (linkedActivities.length > 0) {
                var firstLinkedActivity = linkedActivities[0];
                var linkedTasksCount = await desiteAPI.getPropertyValue(firstSelectedElement, 'cpCountLinkedTasks', 'xs:int');                
                var activityName = await desiteAPI.getPropertyValue(firstLinkedActivity, 'cpName', 'xs:string');
                var startDateUTC = await desiteAPI.getPropertyValue(firstLinkedActivity, 'Start', 'xs:dateTime');
                var finishDateUTC = await desiteAPI.getPropertyValue(firstLinkedActivity, 'Finish', 'xs:dateTime');
                var actualFinishDateUTC = await desiteAPI.getPropertyValue(firstLinkedActivity, 'ActualFinish', 'xs:dateTime');
                startDate = (startDateUTC ? convertUTCDateToDatepickerFormat(startDateUTC) : '');
                finishDate = (finishDateUTC ? convertUTCDateToDatepickerFormat(finishDateUTC) : '');
                actualFinishDate = (actualFinishDateUTC ? convertUTCDateToDatepickerFormat(actualFinishDateUTC) : '');
                $('#inputDateStart').prop('disabled', false);
                $('#inputDateFinish').prop('disabled', false);
                $('#inputDateActualFinish').prop('disabled', false);
            }else{
                linkedTasksCount = 0;                
                alert(`${gSubdictionary["noActivitiesWereFound"]}`);
            }
        }else {
            alert(`${gSubdictionary["numberOfSelectedElements"]}: ${selectedElements.length}\n${gSubdictionary["scheduling_currentlyOnlySingleElementIsSupported"]}`);            
        }
    }    
    $('#elementName').val(name);
    $('#activityName').val(activityName);
    $('#linkedTasksCount').val(linkedTasksCount);
    $('#inputDateStart').val(startDate);
    $('#inputDateFinish').val(finishDate);
    $('#inputDateActualFinish').val(actualFinishDate);
}

function setDatepickerLanguage(currentLanguage)
{           
    var htmlElements = "#inputDateStart, #inputDateFinish, #inputDateActualFinish";        
    var datepickerOptions = (currentLanguage == "de" ? datepickerOptionsGerman() : datepickerOptionsEnglish())    
    $(htmlElements).datepicker("destroy");
    $(htmlElements).datepicker(datepickerOptions);
}

async function showElementsBasedOnFilter()
{        
    var elementList = await desiteAPI.getAllElements('geometry');
    if (gSelectedLevelName != UNDEFINED_OPTION)
    {
        var levelNameFilter = `"${gSelectedLevelName}"`;
        elementList = await desiteAPI.filterByProperty(elementList, 'cpRevitLevel', 'xs:string', levelNameFilter, true ,'geometry');
    }    
    if (gSelectedCategoryName != UNDEFINED_OPTION)
    {        
        var categoryNameFilter = `"${gSelectedCategoryName}"`;    
        elementList = await desiteAPI.filterByProperty(elementList, 'cpRevitCategory', 'xs:string', categoryNameFilter, true ,'geometry');
    }
    if (gShowOnlyStatus && gCurrentStatus != StatusType.None)
    {
        var materialNameList = [];
        if (gCurrentStatus == StatusType.OnTime || gCurrentStatus == StatusType.All)
            materialNameList.push(gOnTimeMaterialName);
        if (gCurrentStatus == StatusType.Overdue || gCurrentStatus == StatusType.All)
            materialNameList.push(gOverdueMaterialName);
        var materialNameFilter = ''
        for (i in materialNameList)        
            materialNameFilter += ((i == (materialNameList.length - 1)) ? `"${materialNameList[i]}"` : `"${materialNameList[i]}" `);
        elementList = await desiteAPI.filterByProperty(elementList, 'cpMaterialName', 'xs:string', materialNameFilter, false ,'geometry');
    }
    await desiteAPI.showElementsOnly(elementList);
}

async function onLevelDropdownMenuChanged() 
{
    var dropdown = document.getElementById('dropdownLevel');
    var selectedIndex = dropdown.selectedIndex;
    gSelectedLevelName = dropdown.options[selectedIndex].value;
    showElementsBasedOnFilter();    
}

async function onCategoryDropdownMenuChanged()
{
    var dropdown = document.getElementById('dropdownCategory');
    var selectedIndex = dropdown.selectedIndex;
    gSelectedCategoryName = dropdown.options[selectedIndex].value;
    showElementsBasedOnFilter();
}

async function createAndActivateColorSchema() 
{
    var colorSchemaName = "Schedule Status";
    var colorSchemaOptions =
    {
        legend: true
    };
    await desiteAPI.createColorSchemaFromCurrentMaterials(colorSchemaName, true, colorSchemaOptions);
    await desiteAPI.activateColorSchemaByName(colorSchemaName);
}

async function createMaterials()
{
    await desiteAPI.createMaterial('<material ID="' + gOverdueMaterialName + '" name="' + gOverdueMaterialName + '">\
    <diff r="250" b="0" g="0" a="1"/>\
    <amb r="250" b="0" g="0" a="1"/>\
    <spec r="255" b="255" g="255" a="1"/>\
    <trans v="0"/>\
    <shin v="0.5"/>\
    </material>');
    
    await desiteAPI.createMaterial('<material ID="' + gOnTimeMaterialName + '" name="' + gOnTimeMaterialName + '">\
    <diff r="0" b="0" g="130" a="1"/>\
    <amb r="0" b="0" g="130" a="1"/>\
    <spec r="255" b="255" g="255" a="1"/>\
    <trans v="0"/>\
    <shin v="0.5"/>\
    </material>');
}

async function createElementListForTimings()
{
    var timings = 
    {
        onTimeElementList: [],
        overdueElementList: []
    };
    var activityElementList = await desiteAPI.getAllElements('activities');     
    activityElementList = await desiteAPI.filterByProperty(activityElementList, 'Finish', 'xs:dateTime', '*', false ,'activities');
    activityElementList = await desiteAPI.filterByProperty(activityElementList, 'ActualFinish', 'xs:dateTime', '*', false ,'activities');
    for (i in activityElementList)
    {
        var activityElement = activityElementList[i];
        var finishDate = new Date(await desiteAPI.getPropertyValue(activityElement, "Finish", "xs:dateTime"));
        var actualFinishDate = new Date(await desiteAPI.getPropertyValue(activityElement, "ActualFinish", "xs:dateTime"));
        var linkedElementList = await desiteAPI.getLinkedObjects(activityElement, "geometry", "activities");
        for (j in linkedElementList) 
        {
            var linkedElement = linkedElementList[j];
            if (finishDate >= actualFinishDate)
                timings.onTimeElementList.push(linkedElement);
            else if (finishDate < actualFinishDate)
                timings.overdueElementList.push(linkedElement);
        }
    }
    return timings;
}

async function setCurrentStatus(status)
{
    gCurrentStatus = status;
    if (gShowOnlyStatus)
        await showElementsBasedOnFilter();
}

async function applyStatusAll()
{
    var timings = await createElementListForTimings();
    await desiteAPI.resetMaterials(true);
    await desiteAPI.setMaterialToObjects(gOnTimeMaterialName, timings.onTimeElementList);
    await desiteAPI.setMaterialToObjects(gOverdueMaterialName, timings.overdueElementList);
    await createAndActivateColorSchema();
    await setCurrentStatus(StatusType.All);
}

async function applyStatusOnTime()
{
    var timings = await createElementListForTimings();
    await desiteAPI.resetMaterials(true);
    await desiteAPI.setMaterialToObjects(gOnTimeMaterialName, timings.onTimeElementList);
    await createAndActivateColorSchema();
    await setCurrentStatus(StatusType.OnTime);
}

async function applyStatusOverdue()
{
    var timings = await createElementListForTimings();
    await desiteAPI.resetMaterials(true);
    await desiteAPI.setMaterialToObjects(gOverdueMaterialName, timings.overdueElementList);
    await createAndActivateColorSchema();
    await setCurrentStatus(StatusType.Overdue);
}

async function applyStatusNone()
{
    await desiteAPI.resetMaterials(true);
    await setCurrentStatus(StatusType.None);
}

async function onStatusDropdownMenuChanged()
{
    var dropdown = document.getElementById('dropdownStatus');
    var selectedIndex = dropdown.selectedIndex;
    var statusType = dropdown.options[selectedIndex].value;
    if(statusType == StatusType.None)
        applyStatusNone();
    else if(statusType == StatusType.All)
        applyStatusAll();
    else if(statusType == StatusType.Overdue)
        applyStatusOverdue();
    else if(statusType == StatusType.OnTime)
        applyStatusOnTime();
}

async function onShowOnlyStatusClicked()
{
    gShowOnlyStatus = $("#showOnlyStatusCheckbox").is(":checked");
    showElementsBasedOnFilter();
}

async function onShowAll()
{
    document.getElementById('dropdownLevel').selectedIndex = 0;
    gSelectedLevelName = UNDEFINED_OPTION;
    document.getElementById('dropdownCategory').selectedIndex = 0;    
    gSelectedCategoryName = UNDEFINED_OPTION;
    document.getElementById('showOnlyStatusCheckbox').checked = false;
    gShowOnlyStatus = false;
    showElementsBasedOnFilter();
}

async function onDateChanged()
{
    var selectedElementList = await desiteAPI.getSelectedElements();
    if (selectedElementList.length != 1)
        return;
    var firstSelectedElement = selectedElementList[0];
    var linkedActivityList = await desiteAPI.getLinkedObjects(firstSelectedElement ,'activities', 'geometry');
    if (linkedActivityList.length == 0)
        return;
    firstLinkedActivity = linkedActivityList[0];
    if ($('#inputDateStart').val())
    {
        var startDate = convertDatepickerFormatToUTCDate($('#inputDateStart').val(), 'T08:00:00');
        await desiteAPI.setPropertyValue(firstLinkedActivity, "Start", "xs:dateTime", startDate );
    }
    if ($('#inputDateFinish').val())
    {
        var finishDate = convertDatepickerFormatToUTCDate($('#inputDateFinish').val(), 'T17:00:00');
        await desiteAPI.setPropertyValue(firstLinkedActivity, "Finish", "xs:dateTime", finishDate);
    }
    if ($('#inputDateActualFinish').val())
    {
        var actualFinishDate = convertDatepickerFormatToUTCDate($('#inputDateActualFinish').val(), 'T17:00:00');
        await desiteAPI.setPropertyValue(firstLinkedActivity, "ActualFinish", "xs:dateTime", actualFinishDate);
    }
    if (gCurrentStatus == StatusType.All)
        applyStatusAll()
    else if (gCurrentStatus == StatusType.OnTime)
        applyStatusOnTime();
    else if (gCurrentStatus == StatusType.Overdue)
        applyStatusOverdue();
}

async function createActivityHierarchyNameList(activityElement)
{
    var hierarchyNameList = [];
    var activityName = await desiteAPI.getPropertyValue(activityElement, "cpName", "xs:string");
    if (activityName != undefined)
    {
        var parentActivityElement = await desiteAPI.getParent(activityElement);
        if (parentActivityElement)
            hierarchyNameList = hierarchyNameList.concat(await createActivityHierarchyNameList(parentActivityElement));
        hierarchyNameList.push(activityName);
    }
    return hierarchyNameList;
}

async function createExcelTable(activityElementList)
{
    var currentDate = Date();
    var currentDateInDatepickerFormat = convertUTCDateToDatepickerFormat(currentDate);
    var userName = await desiteAPI.getUserName();
    var companyName = "Your Company";
    var projectNumber = "A001";
    var createdBy = "Implemented by DESITE BIM Team &copy";
 
    var excelTable =
    `<table style="font-size: 24; font-family: Arial, Helvetica, sans-serif; table-layout: fixed; margin: 4px" cellpadding="4" cellspacing="2" width="100%">
    <tr><td style="background:#ACC22B" colspan="15">
    <tr><td colspan="2">Company: ${companyName}</td>
    <td>User name: ${userName}</td>
    <td>Date: ${currentDateInDatepickerFormat}</td></tr>
    <tr><td colspan="2">Project number: ${projectNumber}</td></tr>
    <tr><td colspan="15"><hr></td></tr>
    <tr style="font-weight: bold;">
    <td style="background: #f0eeee; text-align: center; border-bottom: 5px;" colspan="1"></td>
    <td style="background: #f0eeee; text-align: center; border-bottom: 5px;" colspan="6">Activity name</td>
    <td style="background: #f0eeee; text-align: center; border-bottom: 5px;">Finish date</td>
    <td style="background: #f0eeee; text-align: center; border-bottom: 5px;">Actual Finish date</td>
    <td style="background: #f0eeee; text-align: center; border-bottom: 5px; width:auto;">Time difference in days</td>
    <td style="background: #f0eeee; text-align: center; border-bottom: 5px;">Status</td>
    </tr>`;

    for (i in activityElementList)
    {
        var activityElement = activityElementList[i];
        var hasNoLinkedObjects = (await desiteAPI.getPropertyValue(activityElement, "cpCountLinkedObjects", "xs:int") == undefined);
        if (hasNoLinkedObjects)
            continue;        
        var finishDate = await desiteAPI.getPropertyValue(activityElement, "Finish", "xs:dateTime");
        var actualFinishDate = await desiteAPI.getPropertyValue(activityElement, "ActualFinish", "xs:dateTime");        
        var finishDateInDatepickerFormat = finishDate ? convertUTCDateToDatepickerFormat(finishDate) : '';     
        var actualFinishDateInDatepickerFormat = actualFinishDate ? convertUTCDateToDatepickerFormat(actualFinishDate) : '';        
        var differenceFinishAndActualFinishInDays = 0;
        if (finishDate && actualFinishDate)
        {
            var convertMillisecondsToDays = 1/(1000 * 60 * 60 * 24);        
            differenceFinishAndActualFinishInDays = convertMillisecondsToDays*(new Date(actualFinishDate) - new Date(finishDate));                    
        }                
        var activityHierarchy = '';
        var hierarchyNameList = await createActivityHierarchyNameList(activityElement);
        for (j in hierarchyNameList)
            activityHierarchy += (j == (hierarchyNameList.length - 1) ? hierarchyNameList[j] : hierarchyNameList[j] + ', ');        
        var isOnTime = (differenceFinishAndActualFinishInDays <= 0);
        var status = (isOnTime ? 'On time' : 'Overdue');
        var statusColor = (isOnTime ? '#008200' : '#c80000');

        excelTable +=
        `<tr>
        <td width='200'/>
        <td colspan="6">${activityHierarchy}</td>
        <td style='text-align:center'>${finishDateInDatepickerFormat}</td>
        <td style='text-align:center'>${actualFinishDateInDatepickerFormat}</td>
        <td style='text-align:center'>${differenceFinishAndActualFinishInDays.toFixed(2)}</td>
        <td style='color:${statusColor}; text-align:center'>${status}</td>
        </tr>`;
    }

    excelTable += 
    `<tr/>
    <tr/>
    <tr><td style="background:#ddd" colspan="15"><i>${createdBy}</i>
    <tr><td style="background:#ACC22B" colspan="15">
    </table>`;
                                                                                                                    
    return excelTable;
}

async function onTableExportClicked()
{    
    var activityElementList = await desiteAPI.getAllElements("activities"); 
    var excelTable = await createExcelTable(activityElementList);
    var filePath = await desiteAPI.getProjectDirectory() + "/StatusReport.xls";
    await desiteAPI.writeFile(filePath, excelTable);
    await desiteAPI.openLink('file:///' + filePath, true);
}